<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Category;
use Illuminate\Http\Request;

class ModelsController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $parent = Category::find($request->input('id'));
        $rows = Category::where(['type' => 'car', 'parent_id' => $parent->id])->orderByDesc('id')->get();
        return view('admin.models.index', compact('rows', 'parent'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create(Request $request)
    {
        $row = Category::find($request->input('id'));
        return view('admin.models.create', compact('row'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'name_ar' => 'required',
            'name_en' => 'required',
            'image' => 'required|image'
        ]);
        $category = new Category();
        $category['name->ar'] = $request['name_ar'];
        $category['name->en'] = $request['name_en'];
        $category['parent_id'] = $request['parent_id'];
        $category['type'] = 'car';
        $category->setImageAttribute();
        $category->save();
        return redirect()->route('models.index', 'id=' . $category->parent_id)->with('message', 'تم اضافه البنك بنجاح');
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $row = Category::find($id);
        return view('admin.models.edit', compact('row'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'name_ar' => 'required',
            'name_en' => 'required',
            'image' => 'image'
        ]);
        $category = Category::find($id);
        $category['name->ar'] = $request['name_ar'];
        $category['name->en'] = $request['name_en'];
        if ($request->hasFile('image')) $category->setImageAttribute();

        $category->update();
        return redirect()->route('models.index', 'id=' . $category->parent_id)->with('message', 'تم تعديل البنك بنجاح');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        Category::destroy($id);
        return redirect()->back()->with('message', 'تم الحذف بنجاح');
    }
}
