<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Resources\ProductResource;
use App\Models\Category;
use App\Models\Favorite;
use App\Models\Product;
use App\Models\ProductImage;
use App\Models\Report;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Lang;
use JWTAuth;
use Tymon\JWTAuth\Exceptions\JWTException;


class ProductsController extends Controller
{

    public function single_product(Request $request, $id)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $data = new ProductResource(Product::find($id));
        return response()->json(['value' => 'true', 'data' => $data, 'code' => 200]);
    }

    public function similar_products(Request $request, $id)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $products = Product::whereCategoryId($id)->take(2)->get();
        $data = ProductResource::collection($products);
        return response()->json(['value' => 'true', 'data' => $data, 'code' => 200]);
    }

    public function report_product(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'reason_id' => 'required|exists:report_reasons,id',
            'product_id' => 'required|exists:products,id'
        ]);
        if ($validator->passes()) {
            $report = new Report();
            $report->reason_id = $request->reason_id;
            $report->product_id = $request->product_id;
            $report->user_id = $user->id;
            $report->save();
            return response()->json(['value' => 'true', 'data' => Lang::get('data.report_done'), 'code' => 200]);
        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
    }

    public function add_favourite(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);

        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'product_id' => 'required|exists:products,id'
        ]);
        if ($validator->passes()) {
            $fav = Favorite::where(['user_id' => $user->id, 'product_id' => $request->product_id])->first();
            if ($fav) {
                $fav->delete();
                return response()->json(['value' => 'true', 'data' => \Lang::get('data.delete_favourite'), 'code' => 200]);
            } else {
                $add = new Favorite();
                $add->user_id = $user->id;
                $add->product_id = $request->product_id;
                $add->save();
                return response()->json(['value' => 'true', 'data' => \Lang::get('data.add_favourite'), 'code' => 200]);
            }
        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
    }


       public function product_filter(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (request()->header('Authorization')) {
            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
            }
        }
        $category_id = (integer)$request->category_id;

        if ($category_id === 0) {
            $validator = \Validator::make($request->all(), [
                'type' => 'required|in:car,building',
                'flag' => 'required|in:rent,ownership',
                'city_id' => 'required'
            ]);
        } else {
            $validator = \Validator::make($request->all(), [
                'category_id' => 'required',
                'city_id' => 'required'
            ]);
        }

        if ($validator->passes()) {
            $category_id = (integer)$request->category_id;

            $city_id = (integer)$request->city_id;
            $from = (integer)$request->from;
            $to = (integer)$request->to;

            if ($city_id === 0 && $from && $to && $request->type) {
                $products = Product::where(['type' => $request->type, 'flag' => $request->flag])->whereBetween('price', [$from, $to])->get();
            } elseif ($city_id !== 0 && !$from && !$to && $request->type) {
                $products = Product::where(['type' => $request->type, 'flag' => $request->flag, 'city_id' => $city_id])->get();
            } elseif ($city_id !== 0 && $from && $to && $request->type) {
                $products = Product::where(['type' => $request->type, 'flag' => $request->flag, 'city_id' => $city_id])->whereBetween('price', [$from, $to])->get();
            } elseif ($city_id !== 0 && $category_id !== 0 && $from && $to) {
                $categories = Category::whereParentId($category_id)->pluck('id');
                $all_categories = $categories->merge($category_id);
                $products = Product::where(['city_id' => $city_id])->whereIn('category_id', $all_categories)->whereBetween('price', [$from, $to])->get();
            } elseif ($city_id === 0 && $category_id !== 0 && !$from && !$to) {
                $categories = Category::whereParentId($category_id)->pluck('id');
                $all_categories = $categories->merge($category_id);
                $products = Product::whereIn('category_id', $all_categories)->get();
            } elseif ($city_id === 0 && $category_id !== 0 && $from && $to) {
                $categories = Category::whereParentId($category_id)->pluck('id');
                $all_categories = $categories->merge($category_id);
                $products = Product::whereIn('category_id', $all_categories)->whereBetween('price', [$from, $to])->get();
            } else {
                $products = Product::where(['type' => $request->type, 'flag' => $request->flag])->get();
            }

            return ProductResource::collection($products)->additional(['value' => true, 'code' => 200]);
        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
    }


    public function add_product(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (request()->header('Authorization')) {
            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
            }
        }

        $validator = \Validator::make($request->all(), [
            'title' => 'required',
            'type' => 'required|in:car,building',
            'flag' => 'required|in:rent,ownership',
            'city_id' => 'required|exists:cities,id',
            'category_id' => 'required',
            'sub_category_id' => 'required',
            'description' => 'required',
            'images' => 'required',
            'price' => 'required',
            'km' => 'required_if:type,car',
            'year' => 'required_if:type,car',
            'color' => 'required_if:type,car',
            'number_seats' => 'required_if:type,car',
            'fetis' => 'required_if:type,car|in:manual,automatic',
            'size' => 'required_if:type,building',
            'number_room' => 'required_if:type,building',
            'address' => 'required_if:type,building',
            'number_floors' => 'required_if:type,building'
        ]);
        if ($validator->passes()) {
            $category_id = (integer)$request->category_id;
            $sub_category_id = (integer)$request->sub_category_id;
            $product = new Product();
            $product->title = $request->title;
            $product->type = $request->type;
            $product->flag = $request->flag;
            $product->user_id = $user->id;
            $product->code = uniqid();
            $product->city_id = $request->city_id;
            $product->description = $request->description;
            $product->price = $request->price;
            if ($sub_category_id !== 0) {
                $product->category_id = $request->sub_category_id;
            } else {
                $product->category_id = $request->category_id;
            }
            if ($request->type == 'car') {
                $product->km = $request->km;
                $product->year = $request->year;
                $product->color = $request->color;
                $product->number_seats = $request->number_seats;
                $product->fetis = $request->fetis;
            } else {
                $product->size = $request->size;
                $product->number_room = $request->number_room;
                $product->number_floors = $request->number_floors;
                $product->address = $request->address;
            }
            $product->save();
            if ($request->images) {
                $image_array = $request['images'];
                foreach ($image_array as $image) {
                    $images = new ProductImage();
                    $images->product_id = $product->id;
                    $images->setImageAttribute($image);
                    $images->save();
                }
            }
            return response()->json(['value' => true, 'data' => \Lang::get('data.product_added'), 'code' => 200]);
        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
    }


    public function edit_product(Request $request, $id)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (request()->header('Authorization')) {
            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
            }
        }

        $validator = \Validator::make($request->all(), [
            'title' => 'required',
            'type' => 'required|in:car,building',
            'flag' => 'required|in:rent,ownership',
            'city_id' => 'required|exists:cities,id',
            'category_id' => 'required',
            'sub_category_id' => 'required',
            'description' => 'required',
//          'images' => 'required|image',
            'price' => 'required',
            'km' => 'required_if:type,car',
            'year' => 'required_if:type,car',
            'color' => 'required_if:type,car',
            'number_seats' => 'required_if:type,car',
            'fetis' => 'required_if:type,car|in:manual,automatic',
            'size' => 'required_if:type,building',
            'number_room' => 'required_if:type,building',
            'address' => 'required_if:type,building',
            'number_floors' => 'required_if:type,building'
        ]);
        if ($validator->passes()) {
            $category_id = (integer)$request->category_id;
            $sub_category_id = (integer)$request->sub_category_id;
            $product = Product::find($id);
            if ($product->user_id == $user->id) {

                $product->title = $request->title;
                $product->type = $request->type;
                $product->flag = $request->flag;
                $product->user_id = $user->id;
//                $product->code = uniqid();
                $product->city_id = $request->city_id;
                $product->description = $request->description;
                $product->price = $request->price;
                if ($sub_category_id !== 0) {
                    $product->category_id = $request->sub_category_id;
                } else {
                    $product->category_id = $request->category_id;
                }
                if ($request->type == 'car') {
                    $product->km = $request->km;
                    $product->year = $request->year;
                    $product->color = $request->color;
                    $product->number_seats = $request->number_seats;
                    $product->fetis = $request->fetis;
                } else {
                    $product->size = $request->size;
                    $product->number_room = $request->number_room;
                    $product->number_floors = $request->number_floors;
                    $product->address = $request->address;
                }
                $product->update();
                if ($request->images) {
                    $image_array = $request['images'];
                    foreach ($image_array as $image) {
                        $images = new ProductImage();
                        $images->product_id = $product->id;
                        $images->setImageAttribute($image);
                        $images->save();
                    }
                }
                return response()->json(['value' => true, 'data' => \Lang::get('data.product_added'), 'code' => 200]);
            } else {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.not_owner'), 'code' => 401]);
            }
        }
        return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
    }

    public function delete_image_product($id, Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        ProductImage::destroy($id);
        return response()->json(['value' => true, 'data' => \Lang::get('data.deleted'), 'code' => 200]);
    }



    public function delete_product($id, Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $product = Product::find($id);
        if ($product->user_id == $user->id) {
            Product::destroy($id);
            return response()->json(['value' => true, 'data' => \Lang::get('data.deleted'), 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.not_owner'), 'code' => 401]);
        }
    }
}
