<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Resources\ProductResource;
use App\Http\Resources\UserResource;
use App\Models\Token;
use App\User;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Tymon\JWTAuth\Exceptions\JWTException;
use Illuminate\Support\Facades\Validator;
use JWTAuth;
use App\Models\City;

class UsersController extends Controller
{

    public function register(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);

        $validator = Validator::make($request->all(), [
            'name' => 'required|string|max:255',
            'email' => 'required|string|email|max:255|unique:users',
            'mobile' => 'required|string|max:255|unique:users',
            'city_id' => 'required|exists:cities,id',
          
            'password' => 'required|string|min:6|confirmed',
        ]);
        if ($validator->passes()) {
            $user = new User();
            $user['name'] = $request['name'];
            $user['mobile'] = $request['mobile'];
            $user['email'] = $request['email'];
            $user['city_id'] = $request['city_id'];
            $user['password'] = bcrypt($request['password']);
            $user['verification'] = 'false';
            $user['type'] = 'user';
            $user['status'] = 'active';
            if($request['whatsapp']){
            $user['whatsapp'] = $request['whatsapp'];
            }
            $user['address'] = $request['address'];
            if ($request->hasFile('image')) {
                $user->setImageAttribute();
            }
            if ($request->hasFile('image_cover')) {
                $user->setImageCoverAttribute();
            }
            $user->save();
//
////            generate code and send it to user
            $code = rand(1111, 9999);
            $user->code = $code;
            $user->update();
//            // send code sms


            $check = Token::where('token', $request['device_token'])->first();
            if ($check) {
                $check->status = 'connected';
                if ($request['device_type']) {
                    $check->device_type = $request['device_type'];
                }
                $check->update();
            } else {
                Token::create([
                    'user_id' => $user->id,
                    'token' => $request['device_token'],
                    'device_type' => $request['device_type'],
                    'status' => 'connected'
                ]);
            }

            $data = new UserResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function logout(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);

        $validator = \Validator::make($request->all(), [
            'device_token' => 'required|exists:tokens,token'
        ]);
        if ($validator->passes()) {
            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
            }


            $exist_token = Token::where(['token' => $request['device_token']])->first();

            $exist_token->status = 'disconnected';
            $exist_token->update();

            JWTAuth::invalidate(JWTAuth::getToken());
            return response()->json(['value' => true, 'data' => \Lang::get('data.logout'), 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }


    }

    public function login(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'mobile' => ['required', 'exists:users,mobile'],
            'password' => 'required',
            'device_token' => 'required',
            'device_type' => 'required|in:ios,android',
        ]);
        if ($validator->passes()) {
            $auth = ['mobile' => $request->input('mobile'), 'password' => $request->input('password'), 'status' => 'active', 'type' => 'user'];
            try {
                if (!$token = JWTAuth::attempt($auth, ['exp' => Carbon::now()->addDays(7300)->timestamp])) {

                    return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_password')], 422);
                }
            } catch (JWTException $e) {
                return response()->json(['value' => false, 'msg' => 'could_not_create_token'], 500);
            }
            $user = User::where('mobile', $request['mobile'])->first();

            $check = Token::where('token', $request['device_token'])->first();
            if ($check) {
                $check->status = 'connected';
                if ($request['device_type']) {
                    $check->device_type = $request['device_type'];
                }
                $check->update();
            } else {
                Token::create([
                    'user_id' => $user->id,
                    'token' => $request['device_token'],
                    'device_type' => $request['device_type'],
                    'status' => 'connected'
                ]);
            }

            $data = new UserResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);


        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function account_verification(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'mobile' => 'required|exists:users,mobile',
            'code' => 'required|exists:users,code'
        ]);
        if ($validator->passes()) {
            $user = User::whereMobile($request['mobile'])->first();
            if (!$user) return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_mobile'), 'code' => 422]);
            if ($user->code != $request['code']) return response()->json(['value' => false, 'msg' => \Lang::get('data.account_verification'), 'code' => 422]);
            $data = new UserResource($user);
            $user->verification = 'true';
            $user->save();
            return response()->json(['value' => true, 'data' => $data, 'msg' => \Lang::get('data.correct_code'), 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function forget_password(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'mobile' => 'required|exists:users,mobile'
        ]);
        if ($validator->passes()) {

            $user = User::whereMobile($request['mobile'])->first();
            $code = rand(1111, 9999);
            $user->code = $code;
            $user->update();
            // mobile code sms
            $data = new UserResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function check_forget_code(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'mobile' => 'required|exists:users,mobile',
            'code' => 'required|exists:users,code'
        ]);
        if ($validator->passes()) {
            $user = User::whereMobile($request['mobile'])->first();
            if (!$user) return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_mobile'), 'code' => 422]);
            if ($user->code != $request['code']) return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_code'), 'code' => 422]);
            $data = new UserResource($user);
            return response()->json(['value' => true, 'data' => $data, 'msg' => \Lang::get('data.correct_code'), 'code' => 200]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function change_password(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'mobile' => 'required|exists:users,mobile',
            'password' => 'required|min:6|confirmed',
            'device_token' => 'required',
            'device_type' => 'required|in:android,ios',
        ]);
        if ($validator->passes()) {

            $user = User::whereMobile($request['mobile'])->first();
            $user->password = bcrypt($request['password']);
            $user->update();
            $check = Token::where('token', $request['device_token'])->first();
            if ($check) {
                $check->status = 'connected';
                if ($request['device_type']) {
                    $check->device_type = $request['device_type'];
                }
                $check->update();
            } else {
                Token::create([
                    'user_id' => $user->id,
                    'token' => $request['device_token'],
                    'device_type' => $request['device_type'],
                    'status' => 'connected'
                ]);
            }
            $data = new UserResource($user);


            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }
    }

    public function update_profile(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'image' => 'image',
            'image_cover' => 'image',
            'email' => 'email|unique:users,email,' . $user->id,
            'mobile' => 'unique:users,mobile,' . $user->id,
            'city_id' => 'exists:cities,id',

        ]);
        if ($validator->passes()) {
            $user->name = $request['name'] ? $request['name'] : $user->name;
            $user->mobile = $request['mobile'] ? $request['mobile'] : $user->mobile;
            $user->email = $request['email'] ? $request['email'] : $user->email;
            $user->city_id = $request['city_id'] ? $request['city_id'] : $user->city_id;
            $user->address = $request['address'] ? $request['address'] : $user->address;
            $user->whatsapp = $request['whatsapp'] ? $request['whatsapp'] : $user->whatsapp;
            $code = rand(1111, 9999);
            $user->code = $code;

            if ($request->password) {
                $user->password = bcrypt($request['password']);
            }
            if ($request->hasFile('image')) {
                $user->setImageAttribute();
            }
            if ($request->hasFile('image_cover')) {
                $user->setImageCoverAttribute();
            }
            $user->update();
            $data = new UserResource($user);
            return response()->json(['value' => true, 'data' => $data, 'code' => 200]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first(), 'code' => 422]);
        }


    }

    public function my_favorite(Request $request, $type)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        if ($type == 'car') {
            $data = $user->favorites()->whereType($type)->paginate(10);
        } elseif ($type == 'building') {
            $data = $user->favorites()->whereType($type)->paginate(10);
        } else {
            $data = $user->favorites()->paginate(10);
        }
        return ProductResource::collection($data)->additional(['value' => true, 'code' => 200]);
    }

    public function my_products(Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $data = $user->products()->paginate();
        return ProductResource::collection($data)->additional(['user' => $user, 'value' => true, 'code' => 200]);
    }


    // jwt

    public function getAuthenticatedUser()
    {
        try {

            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['user_not_found'], 404);
            }

        } catch (Tymon\JWTAuth\Exceptions\TokenExpiredException $e) {

            return response()->json(['token_expired'], $e->getStatusCode());

        } catch (Tymon\JWTAuth\Exceptions\TokenInvalidException $e) {

            return response()->json(['token_invalid'], $e->getStatusCode());

        } catch (Tymon\JWTAuth\Exceptions\JWTException $e) {

            return response()->json(['token_absent'], $e->getStatusCode());

        }

        return response()->json(compact('user'));
    }
}
